﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Catamaran.CollectionGenericObjects
{
    public class ArrayGenericObjects<T> : ICollectionGenericObjects<T>
        where T : class
    {
        private T?[] _collection;

        public int Count => _collection.Length;

        public int SetMaxCount
        {
            set
            {
                if (value > 0)
                {
                    if (_collection.Length > 0)
                    {
                        Array.Resize(ref _collection, value);
                    }
                    else
                    {
                        _collection = new T?[value];
                    }
                }
            }
        }

        public ArrayGenericObjects()
        {
            _collection = Array.Empty<T?>();
        }

        public T? Get(int position)
        {
            if (position >= 0 && position < Count)
            {
                return _collection[position];
            }
            return null;
        }

        public int Insert(T obj)
        {
            for (int i = 0; i < Count; i++)
            {
                if (_collection[i] == null)
                {
                    _collection[i] = obj;
                    return i;
                }
            }
            return -1;
        }

        public int Insert(T obj, int position)
        {
            if (position < Count || position >= 0)
            {
                if (_collection[position] == null)
                {
                    _collection[position] = obj;
                    return position;
                }
                else
                {
                    for (int i = 0; i < Count; i++)
                    {
                        if (_collection[i] == null)
                        {
                            _collection[i] = obj;
                            return i;
                        }
                    }
                }
            }
            
            return -1;
        }

        public T? Remove(int position) 
        {
            if (position > Count || position < 0)
            {
                return null;
            }
            T? obj = _collection[position];
            _collection[position] = null;
            return obj;
        }

    }
}
