﻿using Catamaran.Drawings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Catamaran.CollectionGenericObjects
{
    public class StorageCollection<T>
        where T : class
    {
        readonly Dictionary<string, ICollectionGenericObjects<T>> _storages;

        public List<string> Keys => _storages.Keys.ToList();

        public StorageCollection()
        {
            _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
        }

        public void AddCollection(string name, CollectionType collectionType)
        {
            if (!string.IsNullOrEmpty(name) && !_storages.ContainsKey(name)) 
            {
                switch (collectionType)
                {
                    case CollectionType.None:
                        return;
                    case CollectionType.Array:
                        _storages[name] = new ArrayGenericObjects<T>();
                        return;
                    case CollectionType.List:
                        _storages[name] = new ListGenericObjects<T>();
                        return;
                }
                
            }
        }

        public void DelCollection(string name)
        {
            if (string.IsNullOrEmpty(name) || !_storages.ContainsKey(name)) 
            {
                return;
            }
            _storages.Remove(name);
        }

        public ICollectionGenericObjects<T>? this[string name]
        {
            get
            {
                if (!_storages.ContainsKey(name) || string.IsNullOrEmpty(name))
                {
                    return null;
                }
                return _storages[name];
            }
        }
    }
}
