﻿using Catamaran.Drawings;
using Catamaran.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Catamaran
{
    public partial class FormBoatConfig : Form
    {
        private DrawingBoat? _boat = null;

        public event Action<DrawingBoat>? BoatDelegate;

        public FormBoatConfig()
        {
            InitializeComponent();
            panelRed.MouseDown += panel_MouseDown;
            panelGreen.MouseDown += panel_MouseDown;
            panelBlue.MouseDown += panel_MouseDown;
            panelWhite.MouseDown += panel_MouseDown;
            panelBlack.MouseDown += panel_MouseDown;
            panelGray.MouseDown += panel_MouseDown;
            panelMagenta.MouseDown += panel_MouseDown;
            panelYellow.MouseDown += panel_MouseDown;

            buttonCancel.Click += (events, e) => Close();

        }

        public void AddEvent(Action<DrawingBoat>? boatDelegate)
        {
            BoatDelegate += boatDelegate;
        }

        private void DrawObject()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics g = Graphics.FromImage(bmp);
            _boat?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
            _boat?.SetPosition(5, 5);
            _boat?.DrawTransport(g);
            pictureBoxObject.Image = bmp;
        }

        private void labelObject_MouseDown(object sender, MouseEventArgs e)
        {
            var label = sender as Label;
            label?.DoDragDrop(label?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void panelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void panelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text)?.ToString())
            {
                case "labelSimpleObject":
                    _boat = new DrawingBoat((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelAdvancedObject":
                    _boat = new DrawingCatamaran((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White, Color.Black,
                        checkBoxLeftBobber.Checked, checkBoxRightBobber.Checked, checkBoxSail.Checked);
                    break;
            }
            DrawObject();
        }

        private void panel_MouseDown(object? sender, MouseEventArgs e)
        {
            var panel = sender as Panel;
            panel?.DoDragDrop(panel?.BackColor ?? Color.White, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void labelAdditionalColor_DragEnter(Object sender, DragEventArgs e)
        {
            if (_boat is DrawingCatamaran)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    e.Effect = DragDropEffects.Copy;
                }
                else
                {
                    e.Effect = DragDropEffects.None;
                }
            }
        }

        private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat == null)
            {
                return;
            }
            _boat.EntityBoat?.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }

        private void labelAdditionalColor_DragDrop(Object sender, DragEventArgs e)
        {
            if (_boat?.EntityBoat is EntityCatamaran _catamaran)
            {
                _catamaran.SetAdditionalColor((Color)e.Data?.GetData(typeof(Color)));
            }
            DrawObject();
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            if (_boat != null)
            {
                BoatDelegate?.Invoke(_boat);
                Close();
            }
        }

        private void panelWhite_Paint(object sender, PaintEventArgs e)
        {

        }
    }
}
