﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Catamaran.Drawings;
using Catamaran.Entities;
using Catamaran.MovementStrategy;

namespace Catamaran
{
    public partial class FormCatamaran : Form
    {
        private DrawingBoat? _drawingBoat;

        private AbstractStrategy? _strategy;

        public FormCatamaran()
        {
            InitializeComponent();
            _strategy = null;
        }

        public DrawingBoat SetBoat
        {
            set
            {
                _drawingBoat = value;
                _drawingBoat.SetPictureSize(pictureBox1.Width, pictureBox1.Height);
                StrategyBox.Enabled = true;
                _strategy = null;
                Draw();
            }
        }

        private void Draw()
        {
            if (_drawingBoat == null) return;

            Bitmap bmp = new(pictureBox1.Width, pictureBox1.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawingBoat.DrawTransport(gr);
            pictureBox1.Image = bmp;
        }

        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawingBoat == null) return;

            string name = ((Button)sender)?.Name ?? string.Empty;
            bool result = false;
            switch (name)
            {
                case "buttonUp":
                    result = _drawingBoat.MoveTransport(DirectionType.Up); break;
                case "buttonDown":
                    result = _drawingBoat.MoveTransport(DirectionType.Down); break;
                case "buttonLeft":
                    result = _drawingBoat.MoveTransport(DirectionType.Left); break;
                case "buttonRight":
                    result = _drawingBoat.MoveTransport(DirectionType.Right); break;
            }
            if (result)
            {
                Draw();
            }
        }

        private void FormCatamaran_SizeChanged(object sender, EventArgs e)
        {
            if (_drawingBoat == null) return;


            _drawingBoat.SetPictureSize(pictureBox1.Width, pictureBox1.Height);


            if (_drawingBoat.SetPictureSize(pictureBox1.Width, pictureBox1.Height))
            {
                Draw();
            }



        }

        private void StepButton_Click(object sender, EventArgs e)
        {
            if (_drawingBoat == null)
            {
                return;
            }

            if (StrategyBox.Enabled)
            {
                _strategy = StrategyBox.SelectedIndex switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_strategy == null)
                {
                    return;
                }
                _strategy.SetData(new MoveableBoat(_drawingBoat), pictureBox1.Width, pictureBox1.Height);

            }

            if (_strategy == null)
            {
                return;
            }

            StrategyBox.Enabled = false;
            _strategy.MakeStep();
            Draw();

            if (_strategy.GetStatus() == StrategyStatus.Finish)
            {
                StrategyBox.Enabled = true;
                _strategy = null;
            }
        }
    }
}
